#include <iostream>
#include <fstream>
#include <string>
#include <set>
#include <regex>
#include <boost/filesystem.hpp>
#include <boost/algorithm/string.hpp>

namespace fs = boost::filesystem;

int main(int argc, char const* argv[])
{
	//argv
	// 1 path to environment folder
	// 2 Env name
	// 3 simulation



	//eliminate the _fast or _slow suffix
	std::string simulation_no_suffix;
	{
		std::smatch match;
		std::regex r("([0-9,-]+)(.*)");
		std::string input = argv[3];
		std::regex_match(input, match, r);
		std::cout << match[1] << "\n";
		simulation_no_suffix = match[1];
	}
	std::vector<std::string> stl_files;
	std::set<std::string> outlets_files;
	std::set<std::string> environment_files;

	std::string cad_folder = std::string(argv[1]) + "/" + argv[2] + "/cad_models";
	//get all stl files in directory
	for (const auto& file : fs::directory_iterator(cad_folder)) {
		if (fs::is_regular_file(file) && file.path().extension() == ".stl") {
			stl_files.push_back(file.path().filename().string());
			environment_files.insert(file.path().filename().string());
		}
	}


	//parse the simulation name to figure out the numbers that correspond to the outlets
	std::set<std::string> outlets_number;
	boost::split(outlets_number, simulation_no_suffix, boost::is_any_of("-,"));

	for (const auto& outlet : outlets_number) {
		char buffer[4];
		sprintf(buffer, "%03d", std::atoi(outlet.c_str())); //3 digits, with leading zeroes

		std::regex r("(.*" + std::string(buffer) + ".stl)");
		std::smatch match;
		for (const std::string& str : stl_files) {
			if (std::regex_match(str, match, r)) {
				outlets_files.insert(str);
				environment_files.erase(str);
			}
		}

	}

	std::string outputfolder = std::string(argv[1]) + "/" + argv[2] + "/launch/" + argv[3];
	fs::create_directory(outputfolder);

	std::ofstream preprocessing_outfile(outputfolder + "/" + "GADEN_preprocessing.launch");

	preprocessing_outfile << "<launch>\n"
		<< " <arg name=\"scenario\" default=\"" << argv[2] << "\" />\n"
		<< " <arg name=\"simulation\" default=\"" << argv[3] << "\" />\n\n"

		<< " <node pkg=\"gaden_preprocessing\" type=\"preprocessing\" name=\"preprocessing\" output=\"screen\" required=\"true\">\n"

		<< "     #Cell size\n"
		<< "     <param name=\"cell_size\" value=\"0.1\"/>\n"

		<< "     #CAD models of the enviroment (.stl)\n"
		<< "     <param name=\"number_of_models\" value=\"" << environment_files.size() << "\"/>\n";
	{
		int i = 0;
		for (const auto& str : environment_files) {
			preprocessing_outfile << "     <param name=\"model_" << i << "\" value=\"$(find my_env)/$(arg scenario)/cad_models/" << str << "\"/>\n";
			i++;
		}
	}

	preprocessing_outfile << "\n        #CAD model of the outlets (.stl)\n"
		<< "     <param name=\"number_of_outlet_models\" value=\"" << outlets_files.size() << "\"/>\n";
	{
		int i = 0;
		for (const auto& str : outlets_files) {
			preprocessing_outfile << "     <param name=\"outlets_model_" << i << "\" value=\"$(find my_env)/$(arg scenario)/cad_models/" << str << "\"/>\n";
			i++;
		}
	}

	preprocessing_outfile << "\n        #3D Location of a point in free-space\n"
		<< "     <param name=\"empty_point_x\" value=\"0\"/>\n"
		<< "     <param name=\"empty_point_y\" value=\"0\"/>\n"
		<< "     <param name=\"empty_point_z\" value=\"1\"/>\n\n"

		<< "     <param name=\"floor_height\" value=\"-0.64\"/>\n"
		<< "     <param name=\"worldFile\" value=\"$(find my_env)/$(arg scenario)/launch/ros/stage.world\"/>\n\n"



		<< "     #Wind Data (the node will append _i.csv to the name that is specified here)\n"
		<< "     <param name=\"uniformWind\" value=\"false\"/>\n"
		<< "     <param name=\"wind_files\" value=\"$(find my_env)/$(arg scenario)/wind_simulations/$(arg simulation)/$(arg simulation)\"/>\n\n"

		<< "     #Where to write the output files\n"
		<< "     <param name=\"output_path\" value=\"$(find my_env)/$(arg scenario)\" />\n"
		<< " </node>"
		<< "</launch>";










	std::ofstream filament_sim_outfile(outputfolder + "/" + "GADEN.launch");

	filament_sim_outfile
		<< "<launch>\n"

		<< "   <arg name=\"scenario\" default=\"" << argv[2] << "\" />\n"
		<< "   <arg name=\"simulation\" default=\"" << argv[3] << "\" />\n"
		<< "   <arg name=\"source_location_x\" default=\"" << argv[4] << "\" />\n"
		<< "   <arg name=\"source_location_y\" default=\"" << argv[5] << "\" />\n"
		<< "   <arg name=\"source_location_z\" default=\"" << argv[6] << "\" />\n"
		<< "   <arg name=\"gas_type\" default=\"" << argv[7] << "\" />\n"

		<< "    ### NODE environment (for RVIZ visualization) ###\n"
		<< "    <node pkg=\"gaden_environment\" type=\"environment\" name=\"environment\" output=\"screen\">\n"
		<< "        <param name=\"verbose\" value=\"false\"/>\n"
		<< "        <param name=\"wait_preprocessing\" value=\"false\"/>    	  ### wait for the ok from gaden_preprocessing before running\n"
		<< "        <param name=\"fixed_frame\" value=\"map\"/>\n"
		<< "\n"
		<< "        # Plot CAD models (Collada format .dae)\n"
		<< "        <param name=\"number_of_CAD\" value=\"1\"/>\n"
		<< "        <rosparam subst_value=\"True\">\n"
		<< "            CAD_0: package://my_env/$(arg scenario)/cad_models/$(arg scenario).dae\n"
		<< "            CAD_0_color: [0.52, 0.52, 0.52]\n"
		<< "            \n"
		<< "        </rosparam>\n"
		<< "	            \n"
		<< "        # Plot 3D OccupancyGridmap\n"
		<< "        <param name=\"occupancy3D_data\" value=\"$(find my_env)/$(arg scenario)/OccupancyGrid3D.csv\" />\n"
		<< "\n"
		<< "        # Plot gas sources\n"
		<< "        <param name=\"number_of_sources\" value=\"1\"/>\n"
		<< "        <param name=\"source_0_position_x\" value=\"$(arg source_location_x)\"/>\n"
		<< "        <param name=\"source_0_position_y\" value=\"$(arg source_location_y)\"/>\n"
		<< "        <param name=\"source_0_position_z\" value=\"$(arg source_location_z)\"/>\n"
		<< "        <rosparam>\n"
		<< "            source_0_scale: 0.2\n"
		<< "            source_0_color: [0.0, 1.0, 0.0] \n"
		<< "        </rosparam>\n"
		<< "    </node>\n"
		<< "\n"
		<< "\n"
		<< "    ### Filament Simulator 01ms###\n"
		<< "    <node pkg=\"gaden_filament_simulator\" type=\"filament_simulator\" name=\"filament_simulator\" output=\"screen\" required=\"true\">\n"
		<< "        <param name=\"verbose\" value=\"false\" />\n"
		<< "        <param name=\"wait_preprocessing\" value=\"false\"/>    	 ### wait for the ok from preprocessing before starting the simulation\n"
		<< "	    <param name=\"sim_time\" value=\"1000\" />                    ### [sec] Total time of the gas dispersion simulation\n"
		<< "	    <param name=\"time_step\" value=\"0.1\" />                   ### [sec] Time increment between snapshots. Set aprox = cell_size/max_wind_speed.\n"
		<< "	    <param name=\"num_filaments_sec\" value=\"7\" />            ### Num of filaments released each second\n"
		<< "	    <param name=\"variable_rate\" value=\"true\" />              ### If true the number of released filaments would be random(0,numFilaments_sec)\n"
		<< "        <param name=\"filament_stop_steps\" value=\"0\" />\n"
		<< "        <param name=\"ppm_filament_center\" value=\"10\"/>           ### [ppm] Gas concentration at the center of the 3D gaussian (filament)\n"
		<< "        <param name=\"filament_initial_std\" value=\"10\"/>          ### [cm] Sigma of the filament at t=0-> 3DGaussian shape\n"
		<< "        <param name=\"filament_growth_gamma\" value=\"15\"/>         ### [cm²/s] Growth ratio of the filament_std\n"
		<< "        <param name=\"filament_noise_std\" value=\"0.01\"/>          ### [m] Range of the white noise added on each iteration\n"
		<< "	    <param name=\"gas_type\" value=\"$(arg gas_type)\" />                      ### 0=Ethanol, 1=Methane, 2=Hydrogen, 6=Acetone\n"
		<< "	    <param name=\"temperature\" value=\"298\"/>                  ### [Kelvins]\n"
		<< "	    <param name=\"pressure\" value=\"1\"/>                       ### [Atm]\n"
		<< "	    <param name=\"concentration_unit_choice\" value=\"1\"/>      ### 0=molecules/cm3, 1=ppm (when ppm is used, set temp and pressure accordingly)\n"
		<< "\n"
		<< "	    # In the occupancyGrid.csv file we set: cell_size, num_cells, etc. which come from the CFD wind simulation\n"
		<< "	    <param name=\"occupancy3D_data\" value=\"$(find my_env)/$(arg scenario)/OccupancyGrid3D.csv\" />\n"
		<< "	    <param name=\"fixed_frame\" value=\"map\"/>\n"
		<< "\n"
		<< "	    # WindFlow data (from CFD)\n"
		<< "	    <param name=\"wind_data\" value=\"$(find my_env)/$(arg scenario)/wind_simulations/$(arg simulation)/$(arg simulation)_\" />\n"
		<< "	    <param name=\"wind_time_step\" value=\"1\" />                ### (sec) time increment between Wind snapshots\n"
		<< "	    # Loop options\n"
		<< "        <param name=\"allow_looping\" value=\"true\" />\n"
		<< "        <param name=\"loop_from_step\" value=\"1\" />\n"
		<< "        <param name=\"loop_to_step\" value=\"10\" />\n"
		<< "\n"
		<< "	    # Location of the release point!\n"
		<< "	    <param name=\"source_position_x\" value=\"$(arg source_location_x)\"/>            ### (m)\n"
		<< "	    <param name=\"source_position_y\" value=\"$(arg source_location_y)\"/>            ### (m)\n"
		<< "	    <param name=\"source_position_z\" value=\"$(arg source_location_z)\"/>            ### (m)\n"
		<< "\n"
		<< "	    <param name=\"save_results\" value=\"1\" />                  #1=true, 0=false\n"
		<< "	    <param name=\"results_time_step\" value=\"0.5\" />           #(sec) Time increment between saving state to file\n"
		<< "	    <param name=\"results_min_time\" value=\"0.0\" />            #(sec) Time to start saving results to file\n"
		<< "	    <param name=\"writeConcentrations\" value=\"false\" />\n"
		<< "	    <param name=\"results_location\" value=\"$(find my_env)/$(arg scenario)/gas_simulations/$(arg simulation)\" />\n"
		<< "    </node>\n"
		<< "\n"
		<< "    ### RVIZ ###\n"
		<< "    <node name=\"rviz\" pkg=\"rviz\" type=\"rviz\" args=\"-d $(find my_env)/$(arg scenario)/launch/ros/gaden.rviz\"/>\n"
		<< " \n"
		<< "</launch>\n";

}